﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Ozeki.Media.IPCamera;
using Ozeki.Media.MediaHandlers;
using Ozeki.Media.MediaHandlers.Video;
using Ozeki.Media.MJPEGStreaming;
using Ozeki.Media.Video.Controls;

namespace Basic_CameraViewer
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private VideoViewerWPF _videoViewerWpf;

        private BitmapSourceProvider _provider;

        private IIPCamera _ipCamera;

        private WebCamera _webCamera;

        private MediaConnector _connector;

        private SoftPhone _softPhone;

        private IVideoSender _videoSender;

        public MainWindow()
        {
            InitializeComponent();

            _connector = new MediaConnector();

            _provider = new BitmapSourceProvider();

            _softPhone = new SoftPhone();
            _softPhone.OnRegistrationStateChanged += softPhone_OnRegistrationStateChanged;
            _softPhone.OnCallStateChanged += softPhone_OnCallStateChanged;

            SetVideoViewer();
        }

        void InvokeThread(Action action)
        {
            Dispatcher.BeginInvoke(action);
        }

        void softPhone_OnCallStateChanged(object sender, Ozeki.VoIP.CallStateChangedArgs e)
        {
            InvokeThread(() =>
            {
                CallStateText.Text = e.State.ToString();
            });
        }

        void softPhone_OnRegistrationStateChanged(object sender, Ozeki.VoIP.RegistrationStateChangedArgs e)
        {
            InvokeThread(() =>
            {
                PhoneLineStateText.Text = e.State.ToString();
            });
        }

        private void SetVideoViewer()
        {
            _videoViewerWpf = new VideoViewerWPF
            {
                HorizontalAlignment = HorizontalAlignment.Stretch,
                VerticalAlignment = VerticalAlignment.Stretch,
                Background = Brushes.Black
            };
            CameraBox.Children.Add(_videoViewerWpf);

            _videoViewerWpf.SetImageProvider(_provider);
        }

        #region USB Camera Connect/Disconnect

        private void ConnectUSBCamera_Click(object sender, RoutedEventArgs e)
        {
            _webCamera = WebCamera.GetDefaultDevice();
            if (_webCamera == null) return;
            _connector.Connect(_webCamera, _provider);
            _videoSender = _webCamera;

            _webCamera.Start();
            _videoViewerWpf.Start();
        }

        private void DisconnectUSBCamera_Click(object sender, RoutedEventArgs e)
        {
            _videoViewerWpf.Stop();

            _webCamera.Stop();
            _webCamera.Dispose();

            _connector.Disconnect(_webCamera, _provider);
        }
        #endregion

        #region IP Camera Connect/Disconnect

        private void ConnectIPCamera_Click(object sender, RoutedEventArgs e)
        {
            var host = HostTextBox.Text;
            var user = UserTextBox.Text;
            var pass = Password.Password;

            _ipCamera = IPCameraFactory.GetCamera(host, user, pass);
            if (_ipCamera == null) return;
            _connector.Connect(_ipCamera.VideoChannel, _provider);
            _videoSender = _ipCamera.VideoChannel;

            _ipCamera.Start();
            _videoViewerWpf.Start();
        }

        private void DisconnectIPCamera_Click(object sender, RoutedEventArgs e)
        {
            _videoViewerWpf.Stop();

            _ipCamera.Disconnect();
            _ipCamera.Dispose();

            _connector.Disconnect(_ipCamera.VideoChannel, _provider);
        }

        #endregion

        private void SipRegistrationButton_Click(object sender, RoutedEventArgs e)
        {
            var registration = (bool)registrationRequired.IsChecked;
            var displayname = DisplayNameText.Text;
            var username = UserNameText.Text;
            var authname = AuthNameText.Text;
            var password = PasswordText.Text;
            var domain = DomainText.Text;
            var proxy = ProxyText.Text;

            _softPhone.RegisterSipAccount(registration, displayname, username, authname, password, domain, proxy);
        }

        private void SipUnregistrationButton_Click(object sender, RoutedEventArgs e)
        {
            _softPhone.UnregisterPhoneLine();
        }

        private void CallButton_Click(object sender, RoutedEventArgs e)
        {
            _softPhone.CameraVideo = _videoSender;

            var dial = DialToCallText.Text;
            _softPhone.StartCall(dial);
        }

        private void CallHangUpButton_Click(object sender, RoutedEventArgs e)
        {
            _softPhone.CloseCall();
        }
    }
}