﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Ozeki.Media.MediaHandlers;
using Ozeki.VoIP;
using Ozeki.VoIP.SDK;

namespace Basic_CameraViewer
{
    class SoftPhone
    {
        private ISoftPhone _softphone;
        private IPhoneLine _phoneLine;
        private IPhoneCall _call;

        public IVideoSender CameraVideo { get; set; }

        private MediaConnector _connector;
        private PhoneCallVideoSender _videoSender;

        public event EventHandler<RegistrationStateChangedArgs> OnRegistrationStateChanged;
        public event EventHandler<CallStateChangedArgs> OnCallStateChanged;

        public SoftPhone()
        {
            _softphone = SoftPhoneFactory.CreateSoftPhone(5000, 10000);

            _connector = new MediaConnector();
            _videoSender = new PhoneCallVideoSender();
        }

        public void RegisterSipAccount(bool registrationRequired, string displayname, string name, string registername, string password, string domain, string proxy)
        {
            if (_phoneLine != null)
                _phoneLine.RegistrationStateChanged -= phoneLine_RegistrationStateChanged;

            var account = new SIPAccount(registrationRequired, displayname, name, registername, password, domain, proxy);
            _phoneLine = _softphone.CreatePhoneLine(account);
            _phoneLine.RegistrationStateChanged += phoneLine_RegistrationStateChanged;
            _softphone.RegisterPhoneLine(_phoneLine);
        }

        public void UnregisterPhoneLine()
        {
            _softphone.UnregisterPhoneLine(_phoneLine);
        }

        private void phoneLine_RegistrationStateChanged(object sender, RegistrationStateChangedArgs e)
        {
            var handler = OnRegistrationStateChanged;
            if (handler != null)
                handler(this, e);
        }

        public void StartCall(string dial)
        {
            if (_call != null)
                return;

            if (_phoneLine == null) return;
            var dialParams = new DialParameters(dial) { CallType = CallType.AudioVideo };
            var call = _softphone.CreateCallObject(_phoneLine, dialParams);
            call.CallStateChanged += SingleCall_CallStateChanged;
            ConnectToCall(call);
            call.Start();

            _call = call;
        }

        private void SingleCall_CallStateChanged(object sender, CallStateChangedArgs e)
        {
            var call = sender as ICall;
            if (call == null)
                return;

            var handler = OnCallStateChanged;
            if (handler != null)
                handler(sender, e);

            if (call.CallState.IsCallEnded())
            {
                DisconnectFromCall();
                call.CallStateChanged -= SingleCall_CallStateChanged;
                _call = null;
            }
        }

        private void ConnectToCall(ICall call)
        {
            if (call == null)
                return;

            _videoSender.AttachToCall(call);

            _connector.Connect(CameraVideo, _videoSender);
        }

        private void DisconnectFromCall()
        {
            _videoSender.Detach();

            _connector.Disconnect(CameraVideo, _videoSender);
        }

        public void CloseCall()
        {
            _call.HangUp();
        }
    }
}
